/// <reference types="node" />
import type { Palette } from '@behold/sharp-vibrant/lib/color';
import { OptionDefinition } from 'command-line-args';
export { loadSVG, parseColor } from './helpers';
export interface SqipResult {
    content: Buffer;
    metadata: SqipImageMetadata;
}
export interface SqipCliOptionDefinition extends OptionDefinition {
    description?: string;
    required?: boolean;
    default?: boolean;
}
export interface PluginOptions {
    [key: string]: unknown;
}
export interface PluginResolver {
    name: string;
    options?: PluginOptions;
}
export interface SqipOptions {
    input: string | Buffer;
    outputFileName?: string;
    output?: string;
    silent?: boolean;
    parseableOutput?: boolean;
    plugins?: PluginType[];
    width?: number;
    print?: boolean;
}
interface SqipConfig {
    input: string | Buffer;
    outputFileName?: string;
    output?: string;
    silent?: boolean;
    parseableOutput?: boolean;
    plugins: PluginType[];
    width?: number;
    print?: boolean;
}
export interface SqipImageMetadata {
    originalWidth: number;
    originalHeight: number;
    palette: Palette;
    height: number;
    width: number;
    type: 'unknown' | 'pixel' | 'svg';
    mimeType: string;
    filename: string;
    [key: string]: unknown;
}
export type PluginType = PluginResolver | string;
export interface SqipPluginOptions {
    pluginOptions: PluginOptions;
    options: PluginOptions;
    sqipConfig: SqipConfig;
}
export interface SqipPluginInterface {
    sqipConfig: SqipConfig;
    apply(imageBuffer: Buffer, metadata?: SqipImageMetadata): Promise<Buffer> | Buffer;
}
export declare class SqipPlugin implements SqipPluginInterface {
    sqipConfig: SqipConfig;
    options: PluginOptions;
    static cliOptions: SqipCliOptionDefinition[];
    constructor(options: SqipPluginOptions);
    apply(imageBuffer: Buffer, metadata: SqipImageMetadata): Promise<Buffer> | Buffer;
}
export declare function resolvePlugins(plugins: PluginType[]): Promise<(PluginResolver & {
    Plugin: typeof SqipPlugin;
})[]>;
export declare function sqip(options: SqipOptions): Promise<SqipResult | SqipResult[]>;
export * from './helpers';
export declare const mockedMetadata: SqipImageMetadata;
//# sourceMappingURL=sqip.d.ts.map