"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __exportStar = (this && this.__exportStar) || function(m, exports) {
    for (var p in m) if (p !== "default" && !Object.prototype.hasOwnProperty.call(exports, p)) __createBinding(exports, m, p);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.mockedMetadata = exports.sqip = exports.resolvePlugins = exports.SqipPlugin = exports.parseColor = exports.loadSVG = void 0;
const path_1 = __importDefault(require("path"));
const debug_1 = __importDefault(require("debug"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const sharp_vibrant_1 = __importDefault(require("@behold/sharp-vibrant"));
const color_1 = require("@behold/sharp-vibrant/lib/color");
const sharp_1 = __importDefault(require("sharp"));
const term_img_1 = __importStar(require("term-img"));
const cli_table3_1 = __importDefault(require("cli-table3"));
const chalk_1 = __importDefault(require("chalk"));
const mime_1 = __importDefault(require("mime"));
const helpers_1 = require("./helpers");
var helpers_2 = require("./helpers");
Object.defineProperty(exports, "loadSVG", { enumerable: true, get: function () { return helpers_2.loadSVG; } });
Object.defineProperty(exports, "parseColor", { enumerable: true, get: function () { return helpers_2.parseColor; } });
const debug = (0, debug_1.default)('sqip');
const mainKeys = [
    'filename',
    'originalWidth',
    'originalHeight',
    'width',
    'height',
    'type',
    'mimeType'
];
const PALETTE_KEYS = [
    'Vibrant',
    'DarkVibrant',
    'LightVibrant',
    'Muted',
    'DarkMuted',
    'LightMuted'
];
class SqipPlugin {
    constructor(options) {
        const { sqipConfig } = options;
        this.sqipConfig = sqipConfig || {};
        this.options = {};
    }
    apply(imageBuffer, metadata) {
        console.log(metadata);
        return imageBuffer;
    }
}
exports.SqipPlugin = SqipPlugin;
// Resolves plugins based on a given config
// Array of plugin names or config objects, even mixed.
function resolvePlugins(plugins) {
    return __awaiter(this, void 0, void 0, function* () {
        return Promise.all(plugins.map((plugin) => __awaiter(this, void 0, void 0, function* () {
            if (typeof plugin === 'string') {
                plugin = { name: plugin };
            }
            const { name } = plugin;
            if (!name) {
                throw new Error(`Unable to read plugin name from:\n${JSON.stringify(plugin, null, 2)}`);
            }
            const moduleName = name.indexOf('sqip-plugin-') !== -1 ? name : `sqip-plugin-${name}`;
            try {
                debug(`Loading ${moduleName}`);
                const Plugin = yield Promise.resolve(`${moduleName}`).then(s => __importStar(require(s)));
                return Object.assign(Object.assign({}, plugin), { Plugin: Plugin.default });
            }
            catch (err) {
                console.error(err);
                throw new Error(`Unable to load plugin "${moduleName}". Try installing it via:\n\n npm install ${moduleName}`);
            }
        })));
    });
}
exports.resolvePlugins = resolvePlugins;
function processFile({ filePath, buffer, outputFileName, config }) {
    return __awaiter(this, void 0, void 0, function* () {
        const { output, silent, parseableOutput, print } = config;
        const result = yield processImage({ filePath, buffer, config });
        const { content, metadata } = result;
        let outputPath;
        debug(`Processed ${outputFileName}`);
        // Write result svg if desired
        if (output) {
            try {
                // Test if output path already exists
                const stats = yield fs_extra_1.default.stat(output);
                // Throw if it is a file and already exists
                if (!stats.isDirectory()) {
                    throw new Error(`File ${output} already exists. Overwriting is not yet supported.`);
                }
                outputPath = path_1.default.resolve(output, `${outputFileName}.svg`);
            }
            catch (err) {
                // Output directory or file does not exist. We will create it later on.
                outputPath = output;
            }
            debug(`Writing ${outputPath}`);
            yield fs_extra_1.default.writeFile(outputPath, content);
        }
        // Gather CLI output information
        if (!silent) {
            if (outputPath) {
                console.log(`Stored at: ${outputPath}`);
            }
            // Generate preview
            if (!parseableOutput) {
                // Convert to png for image preview
                const preview = yield (0, sharp_1.default)(Buffer.from(content)).png().toBuffer();
                try {
                    (0, term_img_1.default)(preview, {
                        fallback: () => {
                            // SVG results can still be outputted as string
                            if (metadata.type === 'svg') {
                                console.log(content.toString());
                                return;
                            }
                            // No fallback preview solution yet for non-svg files.
                            console.log(`Unable to render a preview for ${metadata.type} files on this machine. Try using https://iterm2.com/`);
                        }
                    });
                }
                catch (err) {
                    if (err instanceof term_img_1.UnsupportedTerminalError) {
                        if (err.name === 'UnsupportedTerminalError') {
                            throw err;
                        }
                    }
                }
            }
            // Metadata
            const tableConfig = parseableOutput
                ? {
                    chars: {
                        top: '',
                        'top-mid': '',
                        'top-left': '',
                        'top-right': '',
                        bottom: '',
                        'bottom-mid': '',
                        'bottom-left': '',
                        'bottom-right': '',
                        left: '',
                        'left-mid': '',
                        mid: '',
                        'mid-mid': '',
                        right: '',
                        'right-mid': '',
                        middle: ' '
                    },
                    style: { 'padding-left': 0, 'padding-right': 0 }
                }
                : undefined;
            // Figure out which metadata keys to show
            // @todo why is this unused?
            // const allKeys = [...mainKeys, 'palette']
            const mainTable = new cli_table3_1.default(tableConfig);
            mainTable.push(mainKeys);
            mainTable.push(mainKeys.map((key) => String(metadata[key]) || 'can not display'));
            console.log(mainTable.toString());
            // Show color palette
            const paletteTable = new cli_table3_1.default(tableConfig);
            paletteTable.push(PALETTE_KEYS);
            paletteTable.push(PALETTE_KEYS.map((key) => { var _a; return (_a = metadata.palette[key]) === null || _a === void 0 ? void 0 : _a.hex; })
                .filter((hex) => typeof hex === 'string')
                .map((hex) => chalk_1.default.hex(hex)(hex)));
            console.log(paletteTable.toString());
            Object.keys(metadata)
                .filter((key) => ![...mainKeys, 'palette'].includes(key))
                .forEach((key) => {
                console.log(chalk_1.default.bold(`${key}:`));
                console.log(metadata[key]);
            });
            if (metadata.type === 'svg' && print) {
                console.log(`Resulting SVG:\n${result.content.toString()}`);
            }
        }
        return result;
    });
}
function processImage({ filePath, buffer, config }) {
    return __awaiter(this, void 0, void 0, function* () {
        // Extract the palette from the image. We delegate to node-vibrant (which is
        // using jimp internally), and it only supports some image formats. In
        // particular, it does not support WebP and HEIC yet.
        //
        // So we try with the given image buffer, and if the code throws an exception
        // we try again after converting to TIFF. If that fails again we give up.
        const paletteResult = yield (() => __awaiter(this, void 0, void 0, function* () {
            const getPalette = (buffer) => sharp_vibrant_1.default.from(buffer).quality(0).getPalette();
            try {
                return yield getPalette(buffer);
            }
            catch (_a) {
                return getPalette(yield (0, sharp_1.default)(buffer).tiff().toBuffer());
            }
        }))();
        const { name: filename } = path_1.default.parse(filePath);
        const mimeType = mime_1.default.getType(filePath) || 'unknown';
        const metadata = {
            filename,
            mimeType,
            originalWidth: paletteResult.imageDimensions.width,
            originalHeight: paletteResult.imageDimensions.height,
            palette: paletteResult.palette,
            // @todo this should be set by plugins and detected initially
            type: 'unknown',
            width: 0,
            height: 0
        };
        // Load plugins
        const plugins = yield resolvePlugins(config.plugins);
        // Determine output image size
        if (config.width && config.width > 0) {
            // Resize to desired output width
            try {
                buffer = yield (0, sharp_1.default)(buffer).resize(config.width).toBuffer();
                const resizedMetadata = yield (0, sharp_1.default)(buffer).metadata();
                metadata.width = resizedMetadata.width || 0;
                metadata.height = resizedMetadata.height || 0;
            }
            catch (err) {
                throw new Error('Unable to resize');
            }
        }
        else {
            // Fall back to original size, keep image as is
            metadata.width = metadata.originalWidth;
            metadata.height = metadata.originalHeight;
        }
        // Interate through plugins and apply them to last returned image
        for (const { name, options: pluginOptions, Plugin } of plugins) {
            try {
                debug(`Construct ${name}`);
                const plugin = new Plugin({
                    sqipConfig: config,
                    pluginOptions: pluginOptions || {},
                    options: {}
                });
                debug(`Apply ${name}`);
                buffer = yield plugin.apply(buffer, metadata);
            }
            catch (err) {
                console.log(`Error thrown in plugin ${name}.`);
                console.dir({ metadata }, { depth: 3 });
                throw err;
            }
        }
        return { content: buffer, metadata };
    });
}
function sqip(options) {
    return __awaiter(this, void 0, void 0, function* () {
        // Build configuration based on passed options and default options
        const defaultOptions = {
            plugins: [
                { name: 'primitive', options: { numberOfPrimitives: 8, mode: 0 } },
                'blur',
                'svgo',
                'data-uri'
            ],
            width: 300,
            parseableOutput: false,
            silent: true,
            print: false
        };
        const config = Object.assign({}, defaultOptions, options);
        const { input, outputFileName, parseableOutput, silent } = config;
        if (parseableOutput) {
            chalk_1.default.level = 0;
        }
        // Validate configuration
        if (!input || input.length === 0) {
            throw new Error('Please provide an input image, e.g. sqip({ input: "input.jpg" })');
        }
        // If input is a Buffer
        if (Buffer.isBuffer(input)) {
            if (!outputFileName) {
                throw new Error('OutputFileName is required when passing image as buffer');
            }
            return processFile({
                filePath: '-',
                buffer: input,
                outputFileName,
                config
            });
        }
        const files = yield (0, helpers_1.locateFiles)(input);
        debug('Found files:');
        debug(files);
        // Test if all files are accessable
        for (const file of files) {
            try {
                debug('check file ' + file);
                yield fs_extra_1.default.access(file, fs_extra_1.default.constants.R_OK);
            }
            catch (err) {
                throw new Error(`Unable to read file ${file}`);
            }
        }
        // Iterate over all files
        const results = [];
        for (const filePath of files) {
            // Apply plugins to files
            if (!silent) {
                console.log(`Processing: ${filePath}`);
            }
            else {
                debug(`Processing ${filePath}`);
            }
            const buffer = yield fs_extra_1.default.readFile(filePath);
            const result = yield processFile({
                filePath,
                buffer,
                outputFileName: outputFileName || path_1.default.parse(filePath).name,
                config
            });
            results.push(result);
        }
        debug(`Finished`);
        // Return as array when input was array or results is only one file
        if (Array.isArray(input) || results.length === 0) {
            return results;
        }
        return results[0];
    });
}
exports.sqip = sqip;
__exportStar(require("./helpers"), exports);
exports.mockedMetadata = {
    filename: 'mocked',
    mimeType: 'image/mocked',
    width: 1024,
    height: 640,
    type: 'svg',
    originalHeight: 1024,
    originalWidth: 640,
    palette: {
        DarkMuted: new color_1.Swatch([4, 2, 0], 420),
        DarkVibrant: new color_1.Swatch([4, 2, 1], 421),
        LightMuted: new color_1.Swatch([4, 2, 2], 422),
        LightVibrant: new color_1.Swatch([4, 2, 3], 423),
        Muted: new color_1.Swatch([4, 2, 4], 424),
        Vibrant: new color_1.Swatch([4, 2, 5], 425)
    }
};
//# sourceMappingURL=sqip.js.map